@@ -1,7 +1,5 @@
 """\
 Tools to write command line interfaces with sub-commands
-See the blog post at http://jimmyg.org/blog/2009/XXX for information
-Public domian, no warranty, use responsibly.
 """
 
 import logging
@@ -45,6 +43,11 @@ class OptionConfigurationError(Exception):
 #            doc_strings['__program__'] = item
 #    return doc_strings
 
+def makeHandler(handler):
+    def make(state=None):
+        return handler
+    return make
+
 def make_man_page(
     program_name, 
     program_description=None, 
@@ -171,28 +174,6 @@ OPTIONS
 }
     return man_page
 
-#def format_result(program_opts, command_opts, command, args):
-#    output = ['program options: ']
-#    for k, vs in program_opts.items():
-#        for v in vs:
-#            if v['value']:
-#                output.append('%s %s, '%(k, v['value']))
-#            else:
-#                output.append('%s, '%(k))
-#    output[-1] = output[-1][:-1]
-#    if command:
-#        output.append('\ncommand options: ')
-#        for k, vs in command_opts.items():
-#            for v in vs:
-#                if v['value']:
-#                    output.append('%s %s, '%(k, v['value']))
-#                else:
-#                    output.append('%s, '%(k))
-#        output[-1] = output[-1][:-1]
-#        output.append('\ncommand:         %s'%command)
-#        output.append('\nargs:            %s'%(', '.join(args)))
-#    return ''.join(output) 
-
 def build_help(options, aliases, program, usage):
     output = []
     output.append('Commands:')
@@ -589,7 +570,7 @@ def _parse_command_line(option_sets, aliases=None):
         )
 
 def handle_program(
-    command_handlers, 
+    command_handler_factories,
     option_sets, 
     aliases, 
     program_options, 
@@ -603,6 +584,7 @@ def handle_program(
     usage: ``%(program)s [PROGRAM_OPTIONS] COMMAND [OPTIONS] ARGS``
 
     Try \`%(program)s COMMAND --help' for help on a specific command."""
+
     # First, are they asking for program help?
     if program_options.has_key('help'):
         # if so provide it no matter what other options are given
@@ -613,7 +595,6 @@ def handle_program(
                 }
             )
         else:
-            raise Exception('ada')
             print strip_docstring(
                 handle_program.__doc__ % {
                     'program': program_name,
@@ -626,7 +607,7 @@ def handle_program(
         # Are they asking for command help:
         if command_options.has_key('help'):
             # if so provide it no matter what other options are given
-            if not command_handlers.has_key(command):
+            if not command_handler_factories.has_key(command):
                 raise getopt.GetoptError('No such command %r'%command)
             if hasattr(help, command):
                 print strip_docstring(
@@ -635,8 +616,9 @@ def handle_program(
                     }
                 )
             else:
+                fn = command_handler_factories[command]()
                 print strip_docstring(
-                    command_handlers[command].__doc__ % {
+                    (fn.__doc__ or 'No help') % {
                         'program': program_name,
                     }
                 )
@@ -665,6 +647,6 @@ def handle_program(
         else:
             logging.basicConfig(level=logging.INFO, format=format)
         # Now handle the command options and arguments
-        command_handlers[command](option_sets, command_options, args)
-
+        command = command_handler_factories[command]()
+        command(option_sets, command_options, args)
 
